/*
 ******************************************************************************

 LIBGPF: <sample> io target pre/post conversion functions

 Copyright (C) 2002 <Author>     [<you@there.com>]

 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
 to deal in the Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute, sublicense,
 and/or sell copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 ******************************************************************************
 */

#include "<sample>_gpf.h"


ssize_t GPF_<sample>_transfernextstreamblock_input(gpf_handle_t handle,
					struct gpf_pipeline_item_t *pipe)
{
	ssize_t rc;
	struct gpf_pipeline_t *pipeline = NULL;
	struct gpf_handle_opio_t *opio = NULL;
	struct <sample>gpf_priv *priv = NULL;

	LIB_ASSERT(handle != NULL);
	LIB_ASSERT(pipe != NULL);

	GPFDPRINT_TARGET("%s:%s:%i: handle: %p, pipe: %p\n",
			DEBUG_INFO, handle, pipe);

	pipeline = handle->pipeline;
	opio = handle->opio_input;
	priv = <sample>_PRIV(opio);

	fprintf(stderr, "read_buf: %i, write_buf: %i\n",
		pipeline->read_buf, pipeline->write_buf);

	LIB_ASSERT(PIPELINE_READBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_READBUFSIZE(pipeline) > 0);
	LIB_ASSERT(PIPELINE_WRITEBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_WRITEBUFSIZE(pipeline) > 0);

	LIB_ASSERT(pipeline->buf_conv == NULL);
	LIB_ASSERT(pipeline->buf_conv_size == 0);
	pipeline->buf_conv = PIPELINE_READBUF(pipeline);
	pipeline->buf_conv_size = PIPELINE_READBUFSIZE(pipeline);

	rc = opio->read(opio,
			PIPELINE_READBUF(pipeline),
			PIPELINE_READBUFSIZE(pipeline));
	fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);

	rc = pipeline->write(pipeline, 1 /* realloc */);
	fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);

	return rc;
}	/* GPF_<sample>_transfernextstreamblock_input */



ssize_t GPF_<sample>_transfernextstreamblock_output(gpf_handle_t handle,
					struct gpf_pipeline_item_t *pipe)
{
	ssize_t rc;
	struct gpf_handle_opio_t *opio = NULL;
	struct gpf_pipeline_t *pipeline = NULL;
	struct <sample>gpf_priv *priv = NULL;

	LIB_ASSERT(handle != NULL);
	LIB_ASSERT(pipe != NULL);

	GPFDPRINT_TARGET("%s:%s:%i: handle: %p, pipe: %p\n",
			DEBUG_INFO, handle, pipe);

	pipeline = handle->pipeline;
	opio = handle->opio_output;
	priv = <sample>_PRIV(opio);

	fprintf(stderr, "read_buf: %i, write_buf: %i\n",
		pipeline->read_buf, pipeline->write_buf);

	LIB_ASSERT(PIPELINE_READBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_READBUFSIZE(pipeline) > 0);
	LIB_ASSERT(PIPELINE_WRITEBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_WRITEBUFSIZE(pipeline) > 0);

	if (pipeline->buf_conv != NULL) {
		/* happens, when it was used
		 * by a pipe sublib
		 */
		free(pipeline->buf_conv);
		pipeline->buf_conv_size = 0;
	}	/* if */

	pipeline->buf_conv = PIPELINE_WRITEBUF(pipeline);
	pipeline->buf_conv_size = PIPELINE_WRITEBUFSIZE(pipeline);

	rc = pipeline->read(pipeline, 1 /* cache */);
	fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);

	rc = opio->write(opio,
			PIPELINE_WRITEBUF(pipeline),
			PIPELINE_WRITEBUFSIZE(pipeline));
	fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);

	return rc;
}	/* GPF_<sample>_transfernextstreamblock_output */





#warning Add your own conversion routines here, you defined in <sample>_gpf.h


/* These two conversion functions are only implementation examples!
 * Change them to your own needs.
 * See the <sample>_gpf.h file for the prototype declaration.
 */

ssize_t GPF_<sample>_conv_<func1>(struct <sample>gpf_priv *priv,
                        void *buf_in, void *buf_out,
                        size_t size_in, size_t size_out)
{
	ssize_t rc = 0;
	size_t conv_bytes;
	char *bufc_in;
	short *bufc_out;

	bufc_in = (char *)buf_in;
	bufc_out = (short *)buf_out;

	LIB_ASSERT(size_in == size_out * 2);

	conv_bytes = size_out;
	while (conv_bytes--) {
		bufc_out[0] = bufc_in[0] << 8 | bufc_in[0];
		bufc_in++;
		bufc_out++;
	}	/* while */

	rc = size_out - conv_bytes;
	return rc;
}	/* GPF_<sample>_conv_<func1> */


ssize_t GPF_<sample>_conv_<func2>(struct <sample>gpf_priv *priv,
			void *buf_in, void *buf_out,
			size_t size_in, size_t size_out)
{
	ssize_t rc = 0;
	size_t conv_bytes;
	short *bufc_in;
	char *bufc_out;

	bufc_in = (short *)buf_in;
	bufc_out = (char *)buf_out;

	LIB_ASSERT(size_in == size_out * 2);

	conv_bytes = size_out;
	while (conv_bytes--) {
		bufc_out[0] = bufc_in[0] >> 8;
		bufc_in++;
		bufc_out++;
	}	/* while */

	rc = size_out - conv_bytes;
	return rc;
}	/* GPF_<sample>_conv_<func2> */
